import os
from typing import Union
from datetime import date, datetime
import pandas as pd

this_dir, this_filename = os.path.split(__file__)
DATA_PATH = os.path.join(this_dir, "data", "regcode.csv")
rc = pd.read_csv(DATA_PATH)

def _check_length(idnum: Union[str, int]) -> str:
    idnum = str(idnum)
    if len(idnum) == 16:
        return idnum
    elif len(idnum) < 16:
        raise ValueError(f'Identification number (NIK) is too short ({len(idnum)} characters), length should be 16')
    elif len(idnum) > 16:
        raise ValueError(f'Identification number (NIK) is too long ({len(idnum)} characters), length should be 16')
    else:
        raise ValueError(f'Identification number (NIK) {len(idnum)} characters, length should be 16')

def _check_birth(idnum: str):
    bcode = idnum[6:12]
    if int(bcode[:2]) > 31:
        bcode = bcode.replace(bcode[:2], str(int(bcode[:2]) - 40).zfill(2), 1)
    try:
        bdtm = datetime.strptime(bcode, "%d%m%y")
        now = datetime.now()
        if bdtm > now:
            bdtm = bdtm.replace(year = bdtm.year - 100)
    except:
        # return NaN if birth date is invalid
        bdtm = float('nan')
    return bdtm

def province(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    prov_code = int(idnum[:2])
    try:
        prov = rc.loc[rc['code'] == prov_code, 'region'].item()
    except:
        prov = float('nan')
    return prov

def city(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    city_code = int(idnum[:4])
    try:
        city = rc.loc[rc['code'] == city_code, 'region'].item()
    except:
        city = float('nan')
    return city

def district(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    dist_code = int(idnum[:6])
    try:
        dist = rc.loc[rc['code'] == dist_code, 'region'].item()
    except:
        dist = float('nan')
    return dist

def gender(idnum: Union[str, int]) -> str:
    idnum = _check_length(idnum)
    gend_code = int(idnum[6:8])
    if gend_code >= 1 and gend_code <= 31:
        gend = "Male"
    elif gend_code >= 41 and gend_code <= 71:
        gend = "Female"
    else:
        raise ValueError("Identification number (NIK) invalid, character[6:8] must be between 1 to 71")
    return gend

def birthdate(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    bdtm = _check_birth(idnum)
    try:
        bdate = bdtm.day
    except:
        bdate = float('nan')
    return bdate

def birthmonth(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    bdtm = _check_birth(idnum)
    try:
        bmonth = bdtm.month
    except:
        bmonth = float('nan')
    return bmonth

def birthyear(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    bdtm = _check_birth(idnum)
    try:
        byear = bdtm.year
    except:
        byear = float('nan')
    return byear

def birthdtm(idnum: Union[str, int]): # birthday in datetime data type
    idnum = _check_length(idnum)
    bdtm = _check_birth(idnum)
    return bdtm

def birthday(idnum: Union[str, int]): # birthday in string data type
    idnum = _check_length(idnum)
    bdtm = _check_birth(idnum)
    try:
        bday = bdtm.strftime("%d %B %Y")
    except:
        bday = float('nan')
    return bday

def age(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    bdtm = _check_birth(idnum)
    today = date.today()
    try:
        age = today.year - bdtm.year - ((today.month, today.day) < (bdtm.month, bdtm.day))
    except:
        age = float('nan')
    return age

def nth_person(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    nth = int(idnum[13:])
    return nth

def all_info(idnum: Union[str, int]) -> dict:
    idnum = _check_length(idnum)
    nik_dict = {'NIK': int(idnum), 'province': province(idnum), 'city': city(idnum), 'district': district(idnum),
            'gender': gender(idnum), 'birth_datetime': birthdtm(idnum), 'birthday': birthday(idnum),
            'age': age(idnum), 'regist_code': nth_person(idnum)}

    return nik_dict


from typing import Union

from nomiden import nik
from nomiden import kk

class NIK:
    """
    A class to represent personal ID number, NIK (Nomor Induk Kependudukan).

    ...

    Parameters
    ----------
        idnum : int or str
            personal ID number (NIK)

    Attributes
    ----------
    province : str
        province of the NIK holder
    city : str
        city of the NIK holder
    gender : str
        gender of the NIK holder
    birthdate : int, 1 to 31
        date of birth (date of the month) of the NIK holder
    birthmonth : int, 1 to 12
        month of birth of the NIK holder
    birthyear : int
        year of birth of the NIK holder
    birthdtm : datetime64
        complete day of birth (datetime64 format) of the NIK holder
    birthday : str
        complete day of birth (str format) of the NIK holder
    age : int
        age of the NIK holder
    nth_person : int
        NIK registration order number
    all_info : dict
        complete information of the NIK holder in dict format
    """
    def __init__(self, idnum: Union[str, int]):
        """
        Constructs all the necessary attributes for the NIK object.

        Parameters
        ----------
            idnum : int or str
                ID number (NIK)
        """
        self.province = nik.province(idnum)
        self.city = nik.city(idnum)
        self.district = nik.district(idnum)
        self.gender = nik.gender(idnum)
        self.birthdate = nik.birthdate(idnum)
        self.birthmonth = nik.birthmonth(idnum)
        self.birthyear = nik.birthyear(idnum)
        self.birthdtm = nik.birthdtm(idnum)
        self.birthday = nik.birthday(idnum)
        self.age = nik.age(idnum)
        self.nth_person = nik.nth_person(idnum)
        self.all_info = nik.all_info(idnum)

class KK:
    """
    A class to represent family ID number, KK (Kartu Keluarga).

    ...

    Parameters
    ----------
        idnum : int or str
            family ID number (KK)

    Attributes
    ----------
    province : str
        province of the family based on KK registered
    city : str
        city of the family based on KK registered
    regdate : int, 1 to 31
        date of registration (date of the month) of KK
    regmonth : int, 1 to 12
        month of registration of KK
    regyear : int
        year of registration of KK
    regdtm : datetime64
        complete day of registration (datetime64 format) of KK
    regday : str
        complete day of registration (str format) of KK
    nth_pub : int
        KK registration order number
    all_info : dict
        complete information of KK in dict format
    """
    def __init__(self, idnum: Union[str, int]):
        """
        Constructs all the necessary attributes for the KK object.

        Parameters
        ----------
            idnum : int or str
                family ID number (KK)
        """
        self.province = kk.province(idnum)
        self.city = kk.city(idnum)
        self.district = kk.district(idnum)
        self.regdate = kk.regdate(idnum)
        self.regmonth = kk.regmonth(idnum)
        self.regyear = kk.regyear(idnum)
        self.regdtm = kk.regdtm(idnum)
        self.regday = kk.regday(idnum)
        self.nth_pub = kk.nth_pub(idnum)
        self.all_info = kk.all_info(idnum)


import os
from typing import Union
from datetime import date, datetime
import pandas as pd

this_dir, this_filename = os.path.split(__file__)
DATA_PATH = os.path.join(this_dir, "data", "regcode.csv")
rc = pd.read_csv(DATA_PATH)

def _check_length(idnum: Union[str, int]) -> str:
    idnum = str(idnum)
    if len(idnum) == 16:
        return idnum
    elif len(idnum) < 16:
        raise ValueError(f'Identification number (KK) is too short ({len(idnum)} characters), length should be 16')
    elif len(idnum) > 16:
        raise ValueError(f'Identification number (KK) is too long ({len(idnum)} characters), length should be 16')
    else:
        raise ValueError(f'Identification number (KK) {len(idnum)} characters, length should be 16')

def _check_reg(idnum: str):
    rcode = idnum[6:12]
    if int(rcode[:2]) > 31:
        rcode = rcode.replace(rcode[:2], str(int(rcode[:2]) - 40).zfill(2), 1)
    try:
        rdtm = datetime.strptime(rcode, "%d%m%y")
        now = datetime.now()
        if rdtm > now:
            rdtm = rdtm.replace(year = rdtm.year - 100)
    except:
        # return NaN if registration date is invalid
        rdtm = float('nan')
    return rdtm

def province(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    prov_code = int(idnum[:2])
    try:
        prov = rc.loc[rc['code'] == prov_code, 'region'].item()
    except:
        prov = float('nan')
    return prov

def city(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    city_code = int(idnum[:4])
    try:
        city = rc.loc[rc['code'] == city_code, 'region'].item()
    except:
        city = float('nan')
    return city

def district(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    dist_code = int(idnum[:6])
    try:
        dist = rc.loc[rc['code'] == dist_code, 'region'].item()
    except:
        dist = float('nan')
    return dist

def regdate(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    rdtm = _check_reg(idnum)
    try:
        rdate = rdtm.day
    except:
        rdate = float('nan')
    return rdate

def regmonth(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    rdtm = _check_reg(idnum)
    try:
        rmonth = rdtm.month
    except:
        rmonth = float('nan')
    return rmonth

def regyear(idnum: Union[str, int]):
    idnum = _check_length(idnum)
    rdtm = _check_reg(idnum)
    try:
        ryear = rdtm.year
    except:
        ryear = float('nan')
    return ryear

def regdtm(idnum: Union[str, int]): # registration day in datetime data type
    idnum = _check_length(idnum)
    rdtm = _check_reg(idnum)
    return rdtm

def regday(idnum: Union[str, int]): # registration day in string data type
    idnum = _check_length(idnum)
    rdtm = _check_reg(idnum)
    try:
        rday = rdtm.strftime("%d %B %Y")
    except:
        rday = float('nan')
    return rday

def nth_pub(idnum: Union[str, int]) -> int:
    idnum = _check_length(idnum)
    nth = int(idnum[13:])
    return nth

def all_info(idnum: Union[str, int]) -> dict:
    kk_dict = {'NIK': int(idnum), 'province': province(idnum), 'city': city(idnum), 'district': district(idnum),
            'regist_datetime': regdtm(idnum), 'regist_day': regday(idnum),
            'regist_code': nth_pub(idnum)}

    return kk_dict